# -*- coding: utf-8 -*-
from odoo import api, registry, SUPERUSER_ID
from odoo.addons.saas_worker.compat import http, db_monodb, route
from odoo.http import request

import json
import werkzeug
from werkzeug.urls import url_encode, url_parse


class AuthOAuthProvider(http.Controller):

    @route('/oauth2/token', auth='none', readonly=False)
    def token(self, client_id, scope, **kw):
        params = {'client_id': client_id, 'scope': scope}

        token = request.env['auth.oauth2.token'].with_user(request.session.uid).get_token(params)
        token['response_type'] = 'token'
        token['client_id'] = client_id
        token['db'] = kw['db']

        url = kw['url'] + '/auth_oauth/oea/?' + url_encode(token)

        return request.redirect(url, local=False)

    @route('/oauth2/auth', auth='none', readonly=False)
    def auth(self, **kw):

        scopes = kw.get('scope', '').split()

        mode = 'signup' if any(s in scopes for s in 'trial download invitation'.split()) else 'login'

        login = request.session.login
        if 'force_login' in scopes:
            mode = 'login' if bool(request.session.uid) else mode
            request.session.logout(keep_db=True)
            scopes.remove('force_login')    # avoid loop
            logged = False
        else:
            logged = bool(request.session.uid)

        if not logged:
            str_scope = ' '.join(scopes)
            login_params = dict(scope=str_scope, mode=mode, login=login or '')
            redirect_params = dict(kw, scope=str_scope)
            if request.session.debug:
                redirect_params['debug'] = 1
                login_params['debug'] = 1

            if 'demo' in scopes:
                login_params['redirect_uri'] = kw.get('redirect_uri')
            if 'skiptheuse' in scopes:
                nothanks_uri = '/oauth2/nothanks?' + url_encode({
                    'client_id': kw.get('client_id', ''),
                    'scope': str_scope,
                    'redirect_uri': kw.get('redirect_uri', ''),
                    'state': kw.get('state', ''),
                })
                login_params['nothanks_uri'] = nothanks_uri
                if 'oneclick' in scopes:
                    return request.redirect(nothanks_uri)
            if kw.get('redirect_uri'):
                ru = url_parse(kw.get('redirect_uri', ''))
                login_params['redirect_hostname'] = '%s://%s' % (ru.scheme, ru.netloc)

            login_params['redirect'] = '/oauth2/auth/?' + url_encode(redirect_params)
            login_url = '/web/%s?%s' % (mode, url_encode(login_params))

            return request.redirect(login_url)

        token = request.env['auth.oauth2.token'].with_user(request.session.uid).get_token(kw)
        return self._redirect(token, kw)

    @route('/oauth2/nothanks', auth='none')
    def nothanks(self, **kw):
        Token = request.env['auth.oauth2.token'].sudo()
        token = Token.get_invalid_token(kw.get('client_id'), kw.get('scope'), '')
        return self._redirect(token, kw)

    def _redirect(self, token, kw):
        if token.get('action'):
            key = request.session.save_action(token['action'])
            u = '/web/?debug' if request.debug else '/web/'
            redirect_url = '%s#sa=%s' % (u, key)
        elif token.get('redirect_uri'):
            redirect_url = token['redirect_uri']
        elif kw.get('redirect_uri'):
            u = url_parse(kw['redirect_uri'])
            params = dict(token, token_type='Bearer', state=kw.get('state', ''))
            if kw.get('response_mode') == 'query':
                query = u.query or ''
                if query:
                    query += "&"
                query += url_encode(params)
                redirect_url = u.replace(query=query).to_url()
            else:
                frag = u.fragment or ''
                if frag:
                    frag += '&'
                frag += url_encode(params)
                redirect_url = u.replace(fragment=frag).to_url()
        else:
            redirect_url = '/account'
        return request.redirect(redirect_url, local=False)

    @route('/oauth2/tokeninfo', auth='none')
    def tokeninfo(self,  access_token=None, **kw):
        # RFC6750 mandates supporting the Header method for transmitting the
        # Bearer token, and only provides for Body Param and Query Param
        # as fallbacks when that method is not applicable.
        # We support them all for backwards-compatibility, but give priority
        # to the Header one.
        auth_header = request.httprequest.headers.get('Authorization')
        if auth_header and auth_header.startswith('Bearer '):
            access_token = auth_header[7:]

        dbname = db_monodb(request)
        if not dbname or not access_token:
            return json.dumps({"error": "No 'dbname' or 'access_token' specified."})
        try:
            r = registry(dbname)
            with r.cursor() as cr:
                env = api.Environment(cr, SUPERUSER_ID, {})
                info = env['auth.oauth2.token'].tokeninfo(access_token)
                return json.dumps(info)
        except Exception as e:
            return json.dumps({"error": str(e)})
